<?php

namespace app\admin\controller\family;

use app\common\controller\Backend;
use app\lib\service\FamilyTreeService;
use think\Db;
use think\Exception;
use think\exception\PDOException;
use think\exception\ValidateException;

/**
 * 
 *
 * @icon fa fa-circle-o
 */
class FamilyPeople extends Backend
{
    
    /**
     * FamilyPeople模型对象
     * @var \app\admin\model\family\FamilyPeople
     */
    protected $model = null;
    protected $searchFields = 'id,name,identity_card,ext3,ext4,ext5';
    protected $selectpageFields = "id,name";

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\family\FamilyPeople;
        $this->view->assign("sexList", $this->model->getSexList());
        $this->view->assign("auditList", $this->model->getAuditList());
        $this->view->assign("statusList", $this->model->getStatusList());
        $this->view->assign("type", $this->request->param("type",1));
    }

    public function import()
    {
        parent::import();
    }

    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */
    

    /**
     * 查看
     */
    public function index()
    {
        //当前是否为关联查询
        $this->relationSearch = false;
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            $list = $this->model
                    ->with("users")
                    ->where($where)
                    ->order($sort, $order)
                    ->paginate($limit);

            foreach ($list as $key=> $val) {
                $val->visible(['id','number','user_id','name','sex','identity_card','photo','ext0','ext1','ext2','ext3','ext4','ext5','createtime','address','audit','status','is_pay','users','type']);
            }
            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }

        $this->assignconfig("type",$this->request->param("type"));
        return $this->view->fetch();
    }

    /**
     * 添加
     */
    public function add()
    {
        if ($this->request->isPost()) {
            $params = $this->request->param("row/a");
            if ($params) {
                $params = $this->preExcludeFields($params);

                if ($this->dataLimit && $this->dataLimitFieldAutoFill) {
                    $params[$this->dataLimitField] = $this->auth->id;
                }
                $result = false;
                Db::startTrans();
                try {
                    //是否采用模型验证
                    if ($this->modelValidate) {
                        $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                        $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                        $this->model->validateFailException(true)->validate($validate);
                    }
                    $params['type'] = 2;
                    $params['audit'] = 1;
                    $params['number'] = getNumbersn("user");
                    $params['name']=$params['surname'].$params['firstname'];
                    $result = $this->model->allowField(true)->save($params);
                    Db::commit();
                } catch (ValidateException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (PDOException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (Exception $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                }
                if ($result !== false) {
                    $this->success();
                } else {
                    $this->error(__('No rows were inserted'));
                }
            }
            $this->error(__('Parameter %s can not be empty', ''));
        }
        return $this->view->fetch();
    }

    /**
     * 编辑
     */
    public function edit($ids = null)
    {
        $row = $this->model->get($ids);

        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds)) {
            if (!in_array($row[$this->dataLimitField], $adminIds)) {
                $this->error(__('You have no permission'));
            }
        }
        if ($this->request->isPost()) {
            $params = $this->request->param("row/a");
            if ($params) {
                $params = $this->preExcludeFields($params);
                $result = false;
                Db::startTrans();
                try {
                    //是否采用模型验证
                    if ($this->modelValidate) {
                        $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                        $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                        $row->validateFailException(true)->validate($validate);
                    }
                    $params['type'] = 2;
                    $params['audit'] = 1;
                    $params['name']=$params['surname'].$params['firstname'];
                    $result = $row->allowField(true)->save($params);
                    Db::commit();
                } catch (ValidateException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (PDOException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (Exception $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                }
                if ($result !== false) {
                    $this->success();
                } else {
                    $this->error(__('No rows were updated'));
                }
            }
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $this->view->assign("row", $row);
        return $this->view->fetch();
    }

    public function getPeople(){
        $data=$this->model->where(['id'=>$this->request->param("id")])->find();
        $this->success('', null, $data);
    }


    public function audit($ids = null)
    {
        $row = $this->model->get($ids);
        if(!$row){
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds)) {
            if (!in_array($row[$this->dataLimitField], $adminIds)) {
                $this->error(__('You have no permission'));
            }
        }
        if ($this->request->isPost()) {
            $params = $this->request->param("row/a");
            if ($params) {

                if ((!$params['intro']  || empty($params['intro'])) && $params['audit'] == 2) {
                    $this->error('请填写操作备注！');
                }
                $params['status']=$params['audit'] ==1?1:0;

                $result = $row->allowField(true)->save($params);

                $this->success('操作成功');
            } else {
                $this->error('数据错误，请重新打开窗口！user_id', $row['user_id']);
            }
        }
        $this->view->assign("row",$row);
        return $this->view->fetch();
    }

}
