<?php

namespace app\admin\controller\family;

use app\common\controller\Backend;
use think\Db;

/**
 * 姓名卡管理
 *
 * @icon fa fa-circle-o
 */
class Namecard extends Backend
{

    /**
     * Namecard模型对象
     * @var \app\admin\model\family\Namecard
     */
    protected $model = null;

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\family\Namecard;
        $this->view->assign("levelList", $this->model->getLevelList());
        $this->view->assign("statusList", $this->model->getStatusList());
    }



    /**
     * 默认生成的控制器所继承的父类中有index/add/edit/del/multi五个基础方法、destroy/restore/recyclebin三个回收站方法
     * 因此在当前控制器中可不用编写增删改查的代码,除非需要自己控制这部分逻辑
     * 需要将application/admin/library/traits/Backend.php中对应的方法复制到当前控制器,然后进行修改
     */


    /**
     * 查看
     *
     * @return string|Json
     * @throws \think\Exception
     * @throws DbException
     */
    public function index()
    {
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if (false === $this->request->isAjax()) {
            return $this->view->fetch();
        }
        //如果发送的来源是 Selectpage，则转发到 Selectpage
        if ($this->request->request('keyField')) {
            return $this->selectpage();
        }
        [$where, $sort, $order, $offset, $limit] = $this->buildparams();
        $list = $this->model
            ->where($where)
            ->order($sort, $order)
            ->paginate($limit);
        $result = ['total' => $list->total(), 'rows' => $list->items()];
        return json($result);
    }

    /**
     * 添加
     *
     * @return string
     * @throws \think\Exception
     */
    public function add()
    {
        if (false === $this->request->isPost()) {
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);

        if ($this->dataLimit && $this->dataLimitFieldAutoFill) {
            $params[$this->dataLimitField] = $this->auth->id;
        }
        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.add' : $name) : $this->modelValidate;
                $this->model->validateFailException()->validate($validate);
            }
            $result = $this->model->allowField(true)->save($params);
            Db::commit();
        } catch (ValidateException|PDOException|Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if ($result === false) {
            $this->error(__('No rows were inserted'));
        }
        $this->success();
    }

    /**
     * 编辑
     *
     * @param $ids
     * @return string
     * @throws DbException
     * @throws \think\Exception
     */
    public function edit($ids = null)
    {
        $row = $this->model->get($ids);
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds) && !in_array($row[$this->dataLimitField], $adminIds)) {
            $this->error(__('You have no permission'));
        }
        if (false === $this->request->isPost()) {
            $this->view->assign('row', $row);
            return $this->view->fetch();
        }
        $params = $this->request->post('row/a');
        if (empty($params)) {
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $params = $this->preExcludeFields($params);
        $result = false;
        Db::startTrans();
        try {
            //是否采用模型验证
            if ($this->modelValidate) {
                $name = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                $row->validateFailException()->validate($validate);
            }
            $result = $row->allowField(true)->save($params);
            Db::commit();
        } catch (ValidateException|PDOException|Exception $e) {
            Db::rollback();
            $this->error($e->getMessage());
        }
        if (false === $result) {
            $this->error(__('No rows were updated'));
        }
        $this->success();
    }

    /**
     * 批量生成卡券
     */
    public function advancecode(){
        if ($this->request->isPost()) {
            $params=$this->request->post('row/a');
            $c =$params['card_number'];
            $m = $this->model->order('id desc')->field('id')->find();
            if($m){
                $offset = $m['id'] + 1;
            }else{
                $offset = 1;
            }
            $cards = [];
            $data['type'] = 1;//二维码类型 1=认领
            $numLen = 12; //卡号编码位数
            $pwdLen = 7;//激活码位数
            $sNumArr = range(0, 9);
            $sPwdArr = array_merge($sNumArr, range('A', 'Z'));
            for ($x = 0; $x < $c; $x++) {
                $tempPwdStr = array();
                for ($i = 0; $i < $pwdLen; $i++) {
                    if(array_rand($sPwdArr) == 'I'){
                        continue;
                    }
                    $tempPwdStr[] = $sPwdArr[array_rand($sPwdArr)];
                }
                $numbers = $offset++;
                $card_number=str_pad($numbers,7,'0',STR_PAD_LEFT );
                $cards[$x]['card_number'] = $card_number;//卡号
                $cards[$x]['activation_code'] = implode('', $tempPwdStr);//激活码
                $cards[$x]['createtime'] = time();//创建时间
                $cards[$x]['family_id'] = $params['family_id'] ? $params['family_id'] : 0;
                $cards[$x]['pid'] = $params['pid'] ? $params['pid'] : 0;
                $cards[$x]['url'] = createQrcode($data,$data['type'],$card_number);
            }
            array_unique($cards, SORT_REGULAR);
            $result = $this->model->insertAll($cards);
            if($result){
                $this->success("操作成功");
            }else{
                $this->error("操作失败");
            }
        }
        return $this->view->fetch();
    }

    /**
     * 绑定码
     */
    public function bindcode($ids = null){
        $row = $this->model->get($ids);
        if ($this->request->isPost()) {
            $params = $this->request->post('row/a');
            $num = $params['card_number'];//绑定多少组卡号编码
            $pid = $params['pid'];
            $start_number = $params['start_number'];//起始卡号
            $where['id'] = ['between', [$start_number, $start_number + $num - 1]];
            $cards = $this->model->where($where)->select();
            foreach ($cards as $key => $val){
                if ($val['pid']) {
                    $this->error("此卡段存在" . $val['card_number'] . "卡券已被绑定,请重新检查");
                }
            }
            $result = $this->model->where($where)->update(['pid' => $pid,'level' =>$params['level']]);
//            $agent = $this->agentmodel->where(['agent_user_id' => $user_id])->field('card_section,type')->find();
            if($result){
//                if($agent['card_section']){
//                    $card_section = $agent['card_section'].','.$start_number.'-'.($start_number + $num - 1);
//                }else{
//                    $card_section = $start_number.'-'.($start_number + $num - 1);
//                }
//                $this->agentmodel->where(['agent_user_id' => $user_id])->update(['card_section' =>$card_section]);
                $this->success("操作成功");
            }else{
                $this->error("操作失败");
            }
        }
        $this->view->assign("row", $row);
        return $this->view->fetch();
    }
}
