<?php

namespace app\admin\controller\fastim;

use addons\fastim\library\Common;
use app\common\controller\Backend;
use think\Db;
use Exception;
use think\exception\PDOException;
use think\exception\ValidateException;

/**
 * 消息记录管理
 *
 * @icon fa fa-circle-o
 */
class Records extends Backend
{

    /**
     * Records模型对象
     * @var \app\admin\model\fastim\Records
     */
    protected $model = null;

    public function _initialize()
    {
        parent::_initialize();
        $this->model = new \app\admin\model\fastim\Records;
        $this->view->assign("typeList", $this->model->getTypeList());
        $this->view->assign("statusList", $this->model->getStatusList());
    }

    public function import()
    {
        parent::import();
    }

    /**
     * 编辑
     */
    public function edit($ids = null)
    {
        $row = $this->model->get($ids);
        if (!$row) {
            $this->error(__('No Results were found'));
        }
        $adminIds = $this->getDataLimitAdminIds();
        if (is_array($adminIds)) {
            if (!in_array($row[$this->dataLimitField], $adminIds)) {
                $this->error(__('You have no permission'));
            }
        }
        if ($this->request->isPost()) {
            $params = $this->request->post("row/a");
            if ($params) {
                $params = $this->preExcludeFields($params);
                $result = false;
                Db::startTrans();
                try {
                    //是否采用模型验证
                    if ($this->modelValidate) {
                        $name     = str_replace("\\model\\", "\\validate\\", get_class($this->model));
                        $validate = is_bool($this->modelValidate) ? ($this->modelSceneValidate ? $name . '.edit' : $name) : $this->modelValidate;
                        $row->validateFailException(true)->validate($validate);
                    }
                    $result = $row->allowField(true)->save($params);
                    Db::commit();
                } catch (ValidateException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (PDOException $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                } catch (Exception $e) {
                    Db::rollback();
                    $this->error($e->getMessage());
                }
                if ($result !== false) {
                    $this->success();
                } else {
                    $this->error(__('No rows were updated'));
                }
            }
            $this->error(__('Parameter %s can not be empty', ''));
        }
        $this->view->assign("row", $row);
        return $this->view->fetch();
    }

    /**
     * 查看
     */
    public function index()
    {
        $this->relationSearch = true;
        //设置过滤方法
        $this->request->filter(['strip_tags', 'trim']);
        if ($this->request->isAjax()) {
            //如果发送的来源是Selectpage，则转发到Selectpage
            if ($this->request->request('keyField')) {
                return $this->selectpage();
            }
            list($where, $sort, $order, $offset, $limit) = $this->buildparams();

            $dbprefix = config('database.prefix');

            $list = $this->model
                ->with([
                    'sender'    => ['fauser', 'admin'],
                    'recipient' => ['fauser', 'admin'],
                    'groupchat'
                ])
                ->where($where)
                ->where($dbprefix . 'fastim_message.type', 'in', ['voice', 'group_chat_notice', 'kbs_list', 'friend_apply', 'group_invitation', 'group_notice', 'group_apply', 'system', 'link', 'file', 'video', 'audio', 'image', 'default'])
                ->order($sort, $order)
                ->paginate($limit);
            foreach ($list as $row) {
                $row->getRelation('groupchat')->visible(['nickname']);
                $row->getRelation('sender')->visible(['nickname']);
                $row->getRelation('recipient')->visible(['nickname']);

                $aNickname  = $row->sender->admin->nickname ?? '';
                $faNickname = $row->sender->fauser->nickname ?? '';
                if ($aNickname || $faNickname) {
                    $row->sender->nickname = $row->sender->nickname . '(' . ($aNickname ? $aNickname : $faNickname) . ')';
                }

                $aNickname  = $row->recipient->admin->nickname ?? '';
                $faNickname = $row->recipient->fauser->nickname ?? '';
                if ($aNickname || $faNickname) {
                    $row->recipient->nickname = $row->recipient->nickname . '(' . ($aNickname ? $aNickname : $faNickname) . ')';
                }

                $row->sender->nickname    = (isset($row->sender->nickname) && $row->sender->nickname) ? $row->sender->nickname : '-';
                $row->recipient->nickname = (isset($row->recipient->nickname) && $row->recipient->nickname) ? $row->recipient->nickname : '-';
            }

            $result = array("total" => $list->total(), "rows" => $list->items());

            return json($result);
        }
        return $this->view->fetch();
    }
}
